@@ -2,7 +2,7 @@
 #include <omp.h>
 #endif
 
-#include <cmath>
+#include <cmath>       /* isnan */
 
 #ifndef ARMA_WRAP_H
 #define ARMA_WRAP_H
@@ -20,5 +20,6 @@
 void checkNAs(Rcpp::NumericVector& vec, std::string var_name);
 void checkNAs(double val, std::string var_name);
 void compareLengths(Rcpp::NumericVector& vec1, Rcpp::NumericVector& vec2);
+void getInfluenceFunctionKM(Rcpp::NumericVector& time, Rcpp::NumericVector& status,arma::vec& atrisk,arma::vec& MC_term2,arma::uvec& sindex,arma::vec& utime);
 
 #endif
